;;; Buid --- The builder for FG42 -*- lexical-binding: t; -*-
;;
;; Copyright (c) 2010-2024 Sameer Rahmani & Contributors
;;
;; Author: Sameer Rahmani <lxsameer@gnu.org>
;; URL: https://devheroes.codes/FG42/FG42
;; Version: 3.0.0
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with thnis program.  If not, see <http://www.gnu.org/licenses/>.
;;
;;; Commentary:
;; Cubes are the building blocks of any `FG42' editor.  Each `cube' is a
;; unit which defines different abilities in a deterministic and idempotent
;; way.  Cubes are composable and a composition of cubes creates an editor.
;;
;;; Code:

(require 'org-element)
(require 'fg42/build/core)


(defmacro comment (&rest _)
  "Ignore the given BODY."
  nil)


(defun ->epoch (date-str)
  "Convert the given DATE-STR to epoch seconds."
  ;; Just because it's easier to deal with date in bash rather than elisp
  (string-to-number
   (shell-command-to-string (concat (format "date -d %s" date-str) " +%s"))))


(defun get-buffer-global-props (prop)
   "Get a plists of global org properties PROP of current buffer."
   (car
    (org-element-map
        (org-element-parse-buffer)
        'keyword
      (lambda (el)
        (when (string-match prop (org-element-property :key el))
          (org-element-property :value el))))))


(defun get-file-global-props (file prop)
  "Return the value of the given global PROP in the given org FILE."
  (with-temp-buffer
    (insert-file-contents file)
    (get-buffer-global-props prop)))

(defmacro with-cwd (path &rest body)
  "Run the BODY with PATH as the current working dir."
  `(let ((cwd (getenv "PWD"))
         (default-directory ,path))
     (cd default-directory)
     (prog1
         ,@body
       (cd cwd))))

(defun fg42/exctract-keywords (file keys)
  "Create a RSS item from the given org FILE for the give KEYS."
  (with-cwd (file-name-directory file)
            (with-temp-buffer
              (insert-file-contents file)
              (org-collect-keywords keys))))


(defun pair-file-with-date (file)
  "Return a pair for the given FILE with date as car and file as cdr."
  (cons
   (->epoch (get-file-global-props file "DATE"))
   file))


(defun get-file-tags (file)
  "Returna list of tags for the given FILE."
  (with-temp-buffer
    (insert-file-contents file)
    (mapcar #'car (org-get-buffer-tags))))


(defun fg42/git-branch ()
  "Return the branch name of the project."
  (let ((default-directory fg42/build-project-root))
    (shell-command-to-string "git branch --show-current")))


(defun fg42/project-version ()
  "Return the current version of the project."
  (let ((default-directory fg42/build-project-root))
    (shell-command-to-string (format "git describe %s" (fg42/git-branch)))))

(defun fg42/version ()
  "Return the current version of the project."
  (let ((default-directory (getenv "FG42_HOME")))
    (shell-command-to-string (format "git describe %s" (fg42/git-branch)))))

(provide 'fg42/build/utils)
;;; utils.el ends here
